// src/fs.ts
import * as fs from "node:fs";
import * as path from "node:path";
import { lookup } from "mrmime";

// src/lib/byte-range.ts
function getByteLength(range, size) {
  let [start, end] = getIndexes(range, size);
  return end - start;
}
function getIndexes(range, size) {
  let start = Math.min(Math.max(0, range.start < 0 ? size + range.start : range.start), size);
  let end = Math.min(Math.max(start, range.end < 0 ? size + range.end : range.end), size);
  return [start, end];
}

// src/lib/lazy-file.ts
var LazyBlob = class extends Blob {
  #content;
  constructor(parts, options) {
    super([], options);
    this.#content = new BlobContent(parts, options);
  }
  /**
   * Returns the blob's contents as an `ArrayBuffer`.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/arrayBuffer)
   */
  async arrayBuffer() {
    return this.#content.arrayBuffer();
  }
  /**
   * Returns the blob's contents as a byte array.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/bytes)
   */
  async bytes() {
    return this.#content.bytes();
  }
  /**
   * The size of the blob in bytes.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/size)
   */
  get size() {
    return this.#content.size;
  }
  /**
   * Returns a new `Blob` that contains the data in the specified range.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/slice)
   */
  slice(start, end, contentType) {
    return this.#content.slice(start, end, contentType);
  }
  /**
   * Returns a stream that can be used to read the blob's contents.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/stream)
   */
  stream() {
    return this.#content.stream();
  }
  /**
   * Returns the blob's contents as a string.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/text)
   */
  async text() {
    return this.#content.text();
  }
};
var LazyFile = class extends File {
  #content;
  constructor(parts, name, options) {
    super([], name, options);
    this.#content = new BlobContent(parts, options);
  }
  /**
   * Returns the file's content as an `ArrayBuffer`.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/arrayBuffer)
   */
  async arrayBuffer() {
    return this.#content.arrayBuffer();
  }
  /**
   * Returns the file's contents as a byte array.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/bytes)
   */
  async bytes() {
    return this.#content.bytes();
  }
  /**
   * The size of the file in bytes.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/size)
   */
  get size() {
    return this.#content.size;
  }
  /**
   * Returns a new `Blob` that contains the data in the specified range.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/slice)
   */
  slice(start, end, contentType) {
    return this.#content.slice(start, end, contentType);
  }
  /**
   * Returns a stream that can be used to read the file's contents.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/stream)
   */
  stream() {
    return this.#content.stream();
  }
  /**
   * Returns the file's contents as a string.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/text)
   */
  async text() {
    return this.#content.text();
  }
};
var BlobContent = class {
  source;
  totalSize;
  range;
  type;
  constructor(parts, options) {
    if (Array.isArray(parts)) {
      this.source = [];
      this.totalSize = 0;
      for (let part of parts) {
        if (part instanceof Blob) {
          this.source.push(part);
          this.totalSize += part.size;
        } else {
          let array;
          if (typeof part === "string") {
            array = new TextEncoder().encode(part);
          } else if (ArrayBuffer.isView(part)) {
            array = new Uint8Array(part.buffer, part.byteOffset, part.byteLength);
          } else {
            array = new Uint8Array(part);
          }
          this.source.push(array);
          this.totalSize += array.byteLength;
        }
      }
    } else {
      this.source = parts;
      this.totalSize = parts.byteLength;
    }
    this.range = options == null ? void 0 : options.range;
    this.type = (options == null ? void 0 : options.type) ?? "";
  }
  async arrayBuffer() {
    return (await this.bytes()).buffer;
  }
  async bytes() {
    let result = new Uint8Array(this.size);
    let offset = 0;
    for await (let chunk of this.stream()) {
      result.set(chunk, offset);
      offset += chunk.length;
    }
    return result;
  }
  get size() {
    return this.range != null ? getByteLength(this.range, this.totalSize) : this.totalSize;
  }
  slice(start = 0, end, contentType = "") {
    let range = this.range != null ? (
      // file.slice().slice() is additive
      { start: this.range.start + start, end: this.range.end + (end ?? 0) }
    ) : { start, end: end ?? this.size };
    return new LazyBlob(this.source, { range, type: contentType });
  }
  stream() {
    if (this.range != null) {
      let [start, end] = getIndexes(this.range, this.totalSize);
      return Array.isArray(this.source) ? streamContentArray(this.source, start, end) : this.source.stream(start, end);
    }
    return Array.isArray(this.source) ? streamContentArray(this.source) : this.source.stream();
  }
  async text() {
    return new TextDecoder("utf-8").decode(await this.bytes());
  }
};
function streamContentArray(content, start = 0, end = Infinity) {
  let index = 0;
  let bytesRead = 0;
  return new ReadableStream({
    async pull(controller) {
      if (index >= content.length) {
        controller.close();
        return;
      }
      let hasPushed = false;
      function pushChunk(chunk) {
        let chunkLength = chunk.byteLength;
        if (!(bytesRead + chunkLength < start || bytesRead >= end)) {
          let startIndex = Math.max(start - bytesRead, 0);
          let endIndex = Math.min(end - bytesRead, chunkLength);
          controller.enqueue(chunk.subarray(startIndex, endIndex));
          hasPushed = true;
        }
        bytesRead += chunkLength;
      }
      async function pushPart(part) {
        if (part instanceof Blob) {
          if (bytesRead + part.size <= start) {
            bytesRead += part.size;
            return;
          }
          for await (let chunk of part.stream()) {
            pushChunk(chunk);
            if (bytesRead >= end) {
              break;
            }
          }
        } else {
          pushChunk(part);
        }
      }
      while (!hasPushed && index < content.length) {
        await pushPart(content[index++]);
        if (bytesRead >= end) {
          controller.close();
          break;
        }
      }
    }
  });
}

// src/fs.ts
function openFile(filename, options) {
  let stats = fs.statSync(filename);
  if (!stats.isFile()) {
    throw new Error(`Path "${filename}" is not a file`);
  }
  let content = {
    byteLength: stats.size,
    stream(start, end) {
      return streamFile(filename, start, end);
    }
  };
  return new LazyFile(content, (options == null ? void 0 : options.name) ?? path.basename(filename), {
    type: (options == null ? void 0 : options.type) ?? lookup(filename),
    lastModified: (options == null ? void 0 : options.lastModified) ?? stats.mtimeMs
  });
}
function streamFile(filename, start = 0, end = Infinity) {
  let read = fs.createReadStream(filename, { start, end: end - 1 }).iterator();
  return new ReadableStream({
    async pull(controller) {
      let { done, value } = await read.next();
      if (done) {
        controller.close();
      } else {
        controller.enqueue(new Uint8Array(value.buffer, value.byteOffset, value.byteLength));
      }
    }
  });
}
function writeFile(to, file) {
  return new Promise(async (resolve) => {
    let writeStream = typeof to === "string" ? fs.createWriteStream(to) : fs.createWriteStream("ignored", { fd: to });
    for await (let chunk of file.stream()) {
      writeStream.write(chunk);
    }
    writeStream.end(() => {
      resolve();
    });
  });
}
export {
  openFile as getFile,
  openFile,
  writeFile
};
